import {
  Body1,
  Button,
  Caption1,
  Card,
  CardFooter,
  CardHeader,
  CardPreview,
} from "@fluentui/react-components";
import {
  DesktopRegular,
  Play16Regular,
  PowerRegular,
} from "@fluentui/react-icons";
import React from "react";
import { VMApi, VMInfo } from "../api/VMApi";
import { AsyncWidget } from "./AsyncWidget";
import { SectionContainer } from "./SectionContainer";

export function VirtualMachinesWidget(): React.ReactElement {
  const [list, setList] = React.useState<VMInfo[] | undefined>();
  const load = async () => {
    setList(await VMApi.GetList());
  };

  return (
    <SectionContainer>
      <AsyncWidget
        loadKey={1}
        load={load}
        loadingMessage="Loading the list virtual machines..."
        errMsg="Failed to load the list of virtual machines!"
        build={() => <VirtualMachinesWidgetInner list={list!} />}
      />
    </SectionContainer>
  );
}

function VirtualMachinesWidgetInner(p: { list: VMInfo[] }): React.ReactElement {
  return (
    <div
      style={{
        display: "flex",
        flexDirection: "row",
        flexWrap: "wrap",
        justifyContent: "center",
      }}
    >
      {p.list.map((v, n) => (
        <VMWidget key={n} vm={v} />
      ))}{" "}
    </div>
  );
}

function VMWidget(p: { vm: VMInfo }): React.ReactElement {
  return (
    <Card
      style={{
        width: "400px",
        maxWidth: "49%",
        height: "220px",
        margin: "10px",
        display: "flex",
        flexDirection: "column",
      }}
    >
      <CardPreview>TODO preview</CardPreview>

      <CardHeader
        image={<DesktopRegular fontSize={32} />}
        header={
          <Body1>
            <b>{p.vm.name}</b>
          </Body1>
        }
        description={<Caption1>STATE TODO</Caption1>}
      />

      <p style={{ flex: 1 }}>{p.vm.description}</p>

      <CardFooter>
        <Button appearance="primary" icon={<Play16Regular />}>
          Start VM
        </Button>
        <Button icon={<PowerRegular />}>Shutdown</Button>
      </CardFooter>
    </Card>
  );
}
